import os
from singer import utils
from singer.catalog import Catalog
from singer import metadata
from tap_surveymonkey.streams import STREAMS


def get_abs_path(path):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), path)

def get_schemas():
    """
    Builds the singer schema and metadata dictionaries.
    """

    schemas = {}
    schemas_metadata = {}

    for stream_name, stream_object in STREAMS.items():

        schema_path = get_abs_path(f"schemas/{stream_name}.json")
        schema = utils.load_json(schema_path)

        if stream_object.replication_method == "INCREMENTAL":
            replication_keys = [stream_object.replication_key]
        else:
            replication_keys = None

        meta = metadata.to_map(metadata.get_standard_metadata(schema=schema,
                                              key_properties=stream_object.key_properties,
                                              replication_method=stream_object.replication_method,
                                              valid_replication_keys=replication_keys,))

        if replication_keys:
            for replication_key in replication_keys:
                meta = metadata.write(meta,
                                      ("properties", replication_key),
                                      "inclusion",
                                      "automatic")

        meta = metadata.to_list(meta)

        schemas[stream_name] = schema
        schemas_metadata[stream_name] = meta

    return schemas, schemas_metadata


def discover():
    """
    Builds the singer catalog for all the streams in the schemas directory.
    """

    schemas, schemas_metadata = get_schemas()
    streams = []


    for schema_name, schema in schemas.items():
        schema_meta = schemas_metadata[schema_name]

        catalog_entry = {
            "stream": schema_name,
            "tap_stream_id": schema_name,
            "schema": schema,
            "metadata": schema_meta,
        }

        streams.append(catalog_entry)

    return Catalog.from_dict({"streams": streams})



import time
import requests
import singer
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

LOGGER = singer.get_logger()


class SurveyMonkeyClient:
    def __init__(self, access_token):
        self.access_token = access_token

    def make_request(self, endpoint, state=None, method="GET", **request_kwargs):
        headers = {
            "Authorization": "bearer %s" % self.access_token,
            "Content-Type": "application/json"
        }
        url = "https://api.surveymonkey.com/v3/%s" % endpoint

        resp = requests.request(method, url, headers=headers, **request_kwargs)
        if resp.status_code == 404:
            return None

        if resp.status_code == 429:
            # check rate limit
            day_remaining = int(resp.headers["X-Ratelimit-App-Global-Day-Remaining"])
            if day_remaining == 0:
                day_reset = int(resp.headers["X-Ratelimit-App-Global-Day-Reset"])
                day_reset += 2
                LOGGER.info(
                    "Sleeping for %d seconds due to SurveyMonkey API rate limit... printing state", day_reset)
                if state:
                    singer.write_state(state)
                time.sleep(day_reset)
                resp = requests.request(method, url, headers=headers, **request_kwargs)

            minute_remaining = int(resp.headers["X-Ratelimit-App-Global-Minute-Remaining"])
            if minute_remaining == 0:
                minute_reset = int(resp.headers["X-Ratelimit-App-Global-Minute-Reset"])
                minute_reset += 2
                LOGGER.info(
                    "Sleeping for %d seconds due to SurveyMonkey API rate limit... printing state", minute_reset)
                if state:
                    singer.write_state(state)
                time.sleep(minute_reset)
                resp = requests.request(method, url, headers=headers, **request_kwargs)

        return resp.json()



#!/usr/bin/env python3
# pylint: disable=E1111
import singer

from tap_surveymonkey.discover import discover
from tap_surveymonkey.sync import sync


REQUIRED_CONFIG_KEYS = ["start_date", "access_token"]
LOGGER = singer.get_logger()


@singer.utils.handle_top_exception(LOGGER)
def main():
    # Parse command line arguments
    args = singer.utils.parse_args(REQUIRED_CONFIG_KEYS)

    # If discover flag was passed, run discovery mode
    if args.discover:
        catalog = discover().dump()
    # Otherwise run in sync mode
    else:
        if args.catalog:
            catalog = args.catalog
        else:
            catalog = discover()

        sync(args.config, args.state, catalog)


if __name__ == "__main__":
    main()



import datetime
import pytz
import singer.utils
from singer import metadata

from tap_surveymonkey.client import SurveyMonkeyClient


LOGGER = singer.get_logger()


DATETIME_PARSE = "%Y-%m-%dT%H:%M:%SZ"
DATETIME_FMT = "%04Y-%m-%dT%H:%M:%S.%fZ"
DATETIME_FMT_MAC = "%Y-%m-%dT%H:%M:%S.%fZ"
SM_DATE_FORMAT = "%Y-%m-%dT%H:%M:%S"
SM_RESPONSE_DATE_FORMAT = "%Y-%m-%dT%H:%M:%S%z"

DEFAULT_PAGE_SIZE = "50"


def strptime(dtime):
    try:
        return datetime.datetime.strptime(dtime, DATETIME_FMT)
    except Exception:
        try:
            return datetime.datetime.strptime(dtime, DATETIME_FMT_MAC)
        except Exception:
            return datetime.datetime.strptime(dtime, DATETIME_PARSE)


def find_max_timestamp(state, stream_id):
    max_time = pytz.utc.localize(datetime.datetime.min)
    for _, last_modified in state["bookmarks"].get(stream_id, {}).items():
        if max_time < pytz.utc.localize(strptime(last_modified)):
            max_time = pytz.utc.localize(strptime(last_modified))
    return max_time


def patch_time_str(obj_dict):
    # The target expects [yyyy-MM-dd"T"HH:mm:ssZ, yyyy-MM-dd"T"HH:mm:ss.[0-9]{1,9}Z] only
    if obj_dict.get("date_modified"):
        time_obj = singer.utils.strptime_to_utc(obj_dict["date_modified"])
        time_str = singer.utils.strftime(time_obj)
        obj_dict["date_modified"] = time_str
    if obj_dict.get("date_created"):
        time_obj = singer.utils.strptime_to_utc(obj_dict["date_created"])
        time_str = singer.utils.strftime(time_obj)
        obj_dict["date_created"] = time_str


class Stream:
    key_properties = None
    replication_method = None
    replication_key = None
    replication_key_from_parent = False # for streams which just return a single record and iterate by its parent, e.g. "SurveyDetails"
    is_sorted = False # indicate whether data is sorted ascending on bookmark value
    mandatory_properties = []

    def __init__(self, stream_id: str, path: str, parent_stream = None):
        self.stream_id = stream_id
        self.path = path
        self._params = {}
        self.parent_stream = parent_stream

    def format_response(self, response):
        return response

    def _modify_record(self, raw_record):
        pass

    def fetch_data(self, client: SurveyMonkeyClient, stream, config, state, parent_row=None, bookmark_value=None):
        path = self.path
        if parent_row:
            for key, value in parent_row.items():
                path = path.replace(f"{{parent_{key}}}", value)

        resp = client.make_request(path, params=None, state=state)
        if not resp:
            raise Exception("Resource not found")
        if resp.get("error"):
            raise Exception(resp)

        self._modify_record(resp)
        yield resp


class PaginatedStream(Stream):
    def get_params(self, stream, config, state, bookmark_value):
        return {
            "per_page": int(config.get("page_size", DEFAULT_PAGE_SIZE)),
            "page": 1
        }

    def format_response(self, response):
        return response.get("data")

    def fetch_data(self, client: SurveyMonkeyClient, stream, config, state, parent_row=None, bookmark_value=None):
        params = self.get_params(stream, config, state, bookmark_value)
        page = 1
        while True:
            if self.stream_id:
                LOGGER.info("Fetching page {} for {}".format(page, self.stream_id))

            path = self.path
            if parent_row:
                for key, value in parent_row.items():
                    path = path.replace(f"{{parent_{key}}}", value)

            resp = client.make_request(path, params=params, state=state)
            if not resp:
                raise Exception("Resource not found")
            if resp.get("error"):
                raise Exception(resp)

            raw_records = self.format_response(resp)

            for raw_record in raw_records:
                self._modify_record(raw_record)
                yield raw_record

            if not resp["links"].get("next"):
                break

            page += 1
            params.update({"page": page})


class SurveyStream(PaginatedStream):
    key_properties = ["id"]

    def get_params(self, stream, config, state, bookmark_value):
        params = super().get_params(stream, config, state, bookmark_value)

        params.update(
            {
                "sort_by": "date_modified",
                "sort_order": "ASC",
                "include": "date_modified"
            })

        if not bookmark_value and config.get("start_date"):
            bookmark_value = config["start_date"]

        if bookmark_value:
            params.update({"start_modified_at": bookmark_value})

        return params

    def fetch_data(self, client: SurveyMonkeyClient, stream, config, state, parent_row=None, bookmark_value=None):
        survey_id = config.get("survey_id")
        if survey_id:
            yield {"id": survey_id}
        else:
            for survey_raw_record in super().fetch_data(client, None, config, state):
                yield survey_raw_record


class Surveys(PaginatedStream):
    key_properties = ["id"]
    replication_method = "INCREMENTAL"
    replication_key = "date_modified"
    is_sorted = True

    OPTIONAL_INCLUDE_FIELDS = [
        "response_count",
        "date_created",
        "date_modified",
        "language",
        "question_count"
    ]

    def get_params(self, stream, config, state, bookmark_value):
        params = super().get_params(stream, config, state, bookmark_value=None)
        include_list = params["include"].split(",") if "include" in params else []

        include_list.append("date_modified")

        # add optional fields
        mdata = metadata.to_map(stream.metadata)
        for optional_field in self.OPTIONAL_INCLUDE_FIELDS:
            if metadata.get(mdata, ("properties", optional_field), "selected"):
                include_list.append(optional_field)

        # remove duplicates
        include_list = list(dict.fromkeys(include_list))

        params.update(
            {
                "sort_by": "date_modified",
                "sort_order": "ASC",
                "include": ",".join(include_list)
            })

        if not bookmark_value and config.get("start_date"):
            bookmark_value = config["start_date"]

        elif bookmark_value:
            bookmark_value_minus_1_min = (datetime.datetime.strptime(bookmark_value, DATETIME_FMT_MAC) - datetime.timedelta(minutes = 1)).strftime(DATETIME_FMT_MAC)
            params.update({"start_modified_at": bookmark_value_minus_1_min})

        return params

    def _modify_record(self, raw_record):
        super()._modify_record(raw_record)

        patch_time_str(raw_record)


class SurveyDetails(Stream):
    key_properties = ["id"]
    replication_method = "INCREMENTAL"
    replication_key = "date_modified"
    replication_key_from_parent = True
    is_sorted = True

    def _modify_record(self, raw_record):
        super()._modify_record(raw_record)

        patch_time_str(raw_record)


class Responses(PaginatedStream):
    key_properties = ["id"]
    replication_method = "INCREMENTAL"
    replication_key = "date_modified"
    is_sorted = True

    def __init__(self, stream_id: str, path: str, parent_stream, simple: bool = False):
        super().__init__(stream_id=stream_id, path=path, parent_stream=parent_stream)
        self.simple = simple

    def get_params(self, stream, config, state, bookmark_value):
        params = super().get_params(stream, config, state, bookmark_value=None)

        params.update(
            {
                "sort_by": "date_modified",
                "sort_order": "ASC"
            })

        if not bookmark_value and config.get("start_date"):
            bookmark_value = config["start_date"]

        if bookmark_value:
            params.update({"start_modified_at": bookmark_value})

        return params

    def _modify_record(self, raw_record):
        super()._modify_record(raw_record)

        patch_time_str(raw_record)


STREAMS = {
    "surveys": Surveys(
        stream_id="surveys",
        path="surveys"),
    "survey_details": SurveyDetails(
        stream_id="survey_details",
        path="surveys/{parent_id}/details",
        parent_stream=SurveyStream(
            stream_id=None,
            path="surveys"
        )),
    "responses": Responses(
        stream_id="responses",
        path="surveys/{parent_id}/responses/bulk",
        parent_stream=SurveyStream(
            stream_id=None,
            path="surveys"
        )),
    "simplified_responses": Responses(
        stream_id="simplified_responses",
        path="surveys/{parent_id}/responses/bulk",
        parent_stream=SurveyStream(
            stream_id=None,
            path="surveys"
        ),
        simple=True),
}



#!/usr/bin/env python
from setuptools import setup

setup(
    name="tap-surveymonkey",
    version="2.0.0",
    description="Singer.io tap for extracting data",
    author="Stitch",
    url="http://singer.io",
    classifiers=["Programming Language :: Python :: 3 :: Only"],
    py_modules=["tap_surveymonkey"],
    install_requires=[
        "singer-python==5.13.0",
        "requests==2.31.0",
    ],
    extras_require={
        'dev': [
            'ipdb',
            'pylint',
            'urllib3==1.26.15'
        ]
    },
    entry_points="""
    [console_scripts]
    tap-surveymonkey=tap_surveymonkey:main
    """,
    packages=["tap_surveymonkey"],
    package_data={
        "schemas": ["tap_surveymonkey/schemas/*.json"]
    },
    include_package_data=True,
)


import singer
from singer.transform import Transformer
from singer import bookmarks, metadata, metrics

from tap_surveymonkey.client import SurveyMonkeyClient
from tap_surveymonkey.streams import STREAMS

LOGGER = singer.get_logger()


def sync(config, state, catalog):
    """ Sync data from tap source """
    access_token = config["access_token"]
    survey_id = config.get("survey_id")
    client = SurveyMonkeyClient(access_token)

    # Loop over selected streams in catalog
    for stream in catalog.get_selected_streams(state):
        stream_object = STREAMS[stream.tap_stream_id]
        mdata = metadata.to_map(stream.metadata)
        raw_schema = stream.schema.to_dict()
        LOGGER.info("Syncing stream: " + stream.tap_stream_id)

        bookmark_column = stream_object.replication_key
        is_sorted = stream_object.is_sorted # indicate whether data is sorted ascending on bookmark value

        # Publish schema to singer.
        singer.write_schema(
            stream_name=stream.tap_stream_id,
            schema=raw_schema,
            key_properties=stream.key_properties,
        )

        with metrics.record_counter(stream.tap_stream_id) as counter:
            max_bookmark = None
            bookmark_value = None
            row_brk_value = None
            with Transformer() as transformer:

                for parent_row in stream_object.parent_stream.fetch_data(client, None, config, state, bookmark_value) if stream_object.parent_stream else [{}]:
                    bookmark_value = bookmarks.get_bookmark(state, stream_object.stream_id, survey_id)

                    if not bookmark_value and not survey_id:
                        bookmark_value = bookmarks.get_bookmark(state, stream_object.stream_id, "full_sync")

                    elif not bookmark_value and stream.tap_stream_id == "surveys":
                        bookmark_value = bookmarks.get_bookmark(state, stream_object.stream_id, "full_sync")

                    for row in stream_object.fetch_data(client, stream, config, state, parent_row=parent_row, bookmark_value=bookmark_value):
                        # write one or more rows to the stream:
                        if not bookmark_value or (bookmark_value and row[bookmark_column] >= bookmark_value):
                            singer.write_record(stream.tap_stream_id, transformer.transform(row, raw_schema, mdata))
                            row_brk_value = row[bookmark_column]
                            counter.increment()

                        if not stream_object.replication_key_from_parent:
                            if is_sorted and stream.tap_stream_id != "surveys":
                                # update bookmark to latest value
                                state = bookmarks.write_bookmark(state, stream.tap_stream_id, row["survey_id"], row[bookmark_column])
                                singer.write_state(state)
                            max_bookmark = max(max_bookmark, row[bookmark_column]) if max_bookmark else row[bookmark_column]
                        
                    if row_brk_value and parent_row and stream_object.replication_key_from_parent:
                        if is_sorted:
                            # update bookmark to latest value
                            state = bookmarks.write_bookmark(state, stream.tap_stream_id, parent_row["id"], parent_row.get(bookmark_column, row_brk_value))
                            singer.write_state(state)
                        max_bookmark = max(max_bookmark, parent_row.get(bookmark_column, row_brk_value)) if max_bookmark else parent_row.get(bookmark_column, row_brk_value)

            if not survey_id or stream.tap_stream_id == "surveys":
                state = bookmarks.write_bookmark(state, stream.tap_stream_id, "full_sync", max_bookmark)
                singer.write_state(state)

            LOGGER.info("Stream: {}, Processed {} records.".format(stream.tap_stream_id, counter.value))
